package com.ads.business.controller;


import com.ads.business.entity.HsayiBrandInfo;
import com.ads.business.model.rep.BrandInfoAddRepVo;
import com.ads.business.model.rep.BrandInfoEditRepVo;
import com.ads.business.model.rep.BrandInfoReqVo;
import com.ads.business.model.resq.BrandInfoDetailRespVo;
import com.ads.business.model.resq.BrandInfoRespVo;
import com.ads.business.model.resq.BrandListRespVo;
import com.ads.business.service.HsayiBrandInfoService;
import com.ads.common.base.BaseController;
import com.ads.common.data.JsonResult;
import com.ads.common.exception.BusinessException;
import com.ads.common.util.UtilConst;
import com.ads.common.util.UtilMessage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 品牌表 前端控制器
 * </p>
 *
 * @author huzhihao
 * @since 2021-12-24
 */
@RestController
@RequestMapping("/api/brand")
@Api(tags = "品牌控制器")
@ApiIgnore
public class HsayiBrandInfoController extends BaseController {

    @Resource
    private HsayiBrandInfoService brandInfoService;

    /**
     * 品牌分页列表
     *
     * @param vo 入参
     * @return 返回结果对象
     */
    @GetMapping("/page")
    @ApiOperation(value = "品牌分页列表", notes = "品牌分页列表", response = BrandInfoRespVo.class)
    public JsonResult page(BrandInfoReqVo vo) {
        return brandInfoService.findPage(vo);
    }

    /**
     * 品牌详情
     *
     * @param brandInfoId 主键ID
     * @return 返回结果对象
     */
    @ApiOperation(value = "品牌详情", notes = "品牌详情", response = BrandInfoDetailRespVo.class)
    @GetMapping("/detail")
    public JsonResult detail(@RequestParam(value = "brandInfoId") Long brandInfoId) {
        return JsonResult.success(brandInfoService.detail(brandInfoId), UtilMessage.GET_MESSAGE_SUCCESS);
    }

    /**
     * 新增
     *
     * @param vo 入参对象
     * @return 返回结果对象
     */
    @ApiOperation(value = "新增", notes = "新增")
    @PostMapping("/add")
    public JsonResult add(@Valid @RequestBody BrandInfoAddRepVo vo) {
        return brandInfoService.add(vo);
    }


    /**
     * 修改
     *
     * @param vo 入参对象
     * @return 返回结果对象
     */
    @ApiOperation(value = "修改", notes = "修改")
    @PutMapping("/edit")
    public JsonResult edit(@Valid @RequestBody BrandInfoEditRepVo vo) {
        return brandInfoService.edit(vo);
    }

    /**
     * 删除
     */
    @DeleteMapping("/{brandInfoIds}")
    @ApiOperation(value = "删除", notes = "删除")
    public JsonResult remove(@PathVariable Long[] brandInfoIds) {
        if (brandInfoIds.length <= 0) {
            throw new BusinessException("请选择要删除的数据");
        }
        return brandInfoService.deleteByIds(brandInfoIds);
    }

    /**
     * 获取审核通过的品牌
     *
     * @return 返回结果对象
     */
    @ApiOperation(value = "获取审核通过的品牌", notes = "获取审核通过的品牌", response = BrandListRespVo.class)
    @GetMapping("/getBrandList")
    public JsonResult getBrandList() {
        LambdaQueryWrapper<HsayiBrandInfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.select(HsayiBrandInfo::getBrand_info_id, HsayiBrandInfo::getBrand_wording);
        wrapper.eq(HsayiBrandInfo::getStatus, UtilConst.authStatus);
        List<HsayiBrandInfo> list = brandInfoService.list(wrapper);
        List<BrandListRespVo> results = Lists.newArrayList();
        list.stream().forEach(t -> {
            BrandListRespVo respVo = new BrandListRespVo();
            respVo.setBrand_info_id(t.getBrand_info_id());
            respVo.setBrand_wording(t.getBrand_wording());
            results.add(respVo);
        });
        return JsonResult.success(results, UtilMessage.GET_MESSAGE_SUCCESS);
    }
}
