package com.ads.business.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ads.business.entity.*;
import com.ads.business.model.BrandImgDto;
import com.ads.business.model.rep.*;
import com.ads.business.service.data.aftersale.AftersaleAddReq;
import com.ads.business.service.data.aftersale.ProductInfo;
import com.ads.business.service.data.brand.BrandAuditPost;
import com.ads.business.service.data.brand.BrandAuditReq;
import com.ads.business.service.data.brand.BrandInfo;
import com.ads.business.service.data.category.CatAuditPost;
import com.ads.business.service.data.category.CatAuditReq;
import com.ads.business.service.data.category.CategoryInfo;
import com.ads.business.service.data.delivery.DeliverySendReq;
import com.ads.business.service.data.order.OrderGetReq;
import com.ads.business.service.data.order.OrderInfoRes;
import com.ads.business.service.data.order.OrderPayReq;
import com.ads.business.service.data.spu.AuditInfo;
import com.ads.business.service.data.spu.Sku;
import com.ads.business.service.data.spu.Spu;
import com.ads.common.constant.CodeConstant;
import com.ads.common.constant.UtilMessage;
import com.ads.common.enums.BrandImgTypeEnum;
import com.ads.common.enums.SpuEditStatusEnum;
import com.ads.common.exception.BusinessException;
import com.ads.common.handler.WeChatHandler;
import com.ads.common.util.RedisUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static com.ads.common.util.UtilMessage.HEAD_IMG_FAILURE;
import static com.ads.common.util.UtilMessage.QUALIFICATION_PIC_FAILURE;

/**
 * @author huzhihao
 * @date 2021-12-26 9:54
 **/
@Slf4j
@Service
public class WechatFacade {

    @Autowired
    private WechatService wechatService;

    @Autowired
    private HsayiCommodityService commodityService;

    @Autowired
    private HsayiSkuService skuService;

    @Autowired
    private HsayiSpuImgService spuImgService;

    @Autowired
    private HsayiCommoditySubmitService commoditySubmitService;

    @Autowired
    private HsayiSkuSubmitService skuSubmitService;

    @Autowired
    private HsayiSpuImgSubmitService spuImgSubmitService;

    @Autowired
    private HsayiAuditInfoService auditInfoService;

    @Autowired
    private HsayiCategoryInfoService categoryInfoService;

    @Autowired
    private HsayiBrandInfoService brandInfoService;

    /**
     * [上传]
     *
     * @param file
     * @return java.lang.String
     * @author huzhihao
     * @date 2021/12/26 10:40
     */
    public String upload(MultipartFile file) {
        return wechatService.upload(file);
    }

    /**
     * [添加商品]
     *
     * @param vo
     * @return com.ads.business.service.data.spu.Spu
     * @author huzhihao
     * @date 2021/12/26 10:40
     */
    @Transactional(rollbackFor = Exception.class)
    public Spu spuAdd(CommodityAddRepVo vo) {
        Spu param = new Spu();
        param.setCreate_time(DateUtil.now());
        BeanUtil.copyProperties(vo, param);
        //设置头图
        param.setHead_img(Arrays.asList(vo.getThumb_img()));
        //设置资质图
        if (StrUtil.isNotBlank(vo.getQualificationImgs())) {
            param.setQualification_pics(Arrays.asList(vo.getQualificationImgs().split(StrUtil.COMMA)));
        }
        //设置sku
        List<Sku> skus = new ArrayList<>();
        Sku sku = new Sku();
        BeanUtil.copyProperties(vo, sku);
        sku.setSale_price(vo.getSale_price().multiply(BigDecimal.valueOf(100)).intValue());
        sku.setMarket_price(vo.getMarket_price().multiply(BigDecimal.valueOf(100)).intValue());
        sku.setOut_sku_id(vo.getOut_product_id());
        skus.add(sku);
        param.setSkus(skus);
        wechatService.spuAdd(param);
        Spu spu = wechatService.spuGet(vo.getOut_product_id());
        validatePic(spu);
        //更新状态
        commoditySubmitService.update(Wrappers.<HsayiCommoditySubmit>lambdaUpdate()
                .eq(HsayiCommoditySubmit::getOut_product_id, vo.getOut_product_id())
                .set(HsayiCommoditySubmit::getStatus, spu.getStatus())
                .set(HsayiCommoditySubmit::getEdit_status, spu.getEdit_status())
                .set(HsayiCommoditySubmit::getCreate_time, DateUtil.parse(spu.getCreate_time(), DatePattern.NORM_DATETIME_PATTERN))
        );
        //插入审核数据
        HsayiAuditInfo auditInfo = new HsayiAuditInfo();
        auditInfo.setStatus(spu.getEdit_status());
        auditInfo.setOut_product_id(vo.getOut_product_id());
        auditInfo.setSubmit_time(DateUtil.now());
        auditInfoService.saveOrUpdate(auditInfo);
        return spu;
    }

    /**
     * [验证图片正确性]
     *
     * @param spu
     * @return void
     * @author huzhihao
     * @date 2021/12/30 17:35
     */
    @Transactional(rollbackFor = Exception.class)
    public void validatePic(Spu spu) {
        List<String> headImgList = spu.getHead_img();
        List<String> qualificationPicList = spu.getQualification_pics();
        for (String headImg : headImgList) {
            if (StrUtil.isBlank(headImg)) {
                wechatService.spuDel(spu.getOut_product_id());
                throw new BusinessException(HEAD_IMG_FAILURE);
            }
        }
        for (String qualificationPic : qualificationPicList) {
            if (StrUtil.isBlank(qualificationPic)) {
                wechatService.spuDel(spu.getOut_product_id());
                throw new BusinessException(QUALIFICATION_PIC_FAILURE);
            }
        }
    }

    /**
     * [添加商品]
     *
     * @param param
     * @return com.ads.business.service.data.spu.Spu
     * @author huzhihao
     * @date 2021/12/26 10:40
     */
    public Spu spuAdd(Spu param) {
        Spu spu = wechatService.spuAdd(param);
        String out_product_id = spu.getOut_product_id();
        return wechatService.spuGet(out_product_id);
    }

    /**
     * [更新商品]
     *
     * @param vo
     * @return com.ads.business.service.data.spu.Spu
     * @author huzhihao
     * @date 2021/12/26 10:40
     */
    @Transactional(rollbackFor = Exception.class)
    public Spu spuUpdate(CommodityEditRepVo vo) {
        Spu param = new Spu();
        BeanUtil.copyProperties(vo, param);
        param.setUpdate_time(DateUtil.now());
        //设置头图
        param.setHead_img(Arrays.asList(vo.getThumb_img()));
        //设置资质图
        if (StrUtil.isNotBlank(vo.getQualificationImgs())) {
            param.setQualification_pics(Arrays.asList(vo.getQualificationImgs().split(StrUtil.COMMA)));
        }
        //设置sku
        List<Sku> skus = new ArrayList<>();
        Sku sku = new Sku();
        BeanUtil.copyProperties(vo, sku);
        sku.setSale_price(vo.getSale_price().multiply(BigDecimal.valueOf(100)).intValue());
        sku.setMarket_price(vo.getMarket_price().multiply(BigDecimal.valueOf(100)).intValue());
        sku.setOut_sku_id(vo.getOut_product_id());
        skus.add(sku);
        param.setSkus(skus);
        wechatService.spuUpdate(param);
        Spu spu = wechatService.spuGet(vo.getOut_product_id());
        validatePic(spu);
        //更新状态
        commoditySubmitService.update(Wrappers.<HsayiCommoditySubmit>lambdaUpdate()
                .eq(HsayiCommoditySubmit::getOut_product_id, vo.getOut_product_id())
                .set(HsayiCommoditySubmit::getStatus, spu.getStatus())
                .set(HsayiCommoditySubmit::getEdit_status, spu.getEdit_status())
                .set(HsayiCommoditySubmit::getCreate_time, DateUtil.parse(spu.getCreate_time(), DatePattern.NORM_DATETIME_PATTERN))
                .set(HsayiCommoditySubmit::getUpdate_time, DateUtil.parse(spu.getUpdate_time(), DatePattern.NORM_DATETIME_PATTERN))
        );
        //更新审核数据
        AuditInfo audit_info = spu.getAudit_info();
        auditInfoService.update(Wrappers.<HsayiAuditInfo>lambdaUpdate()
                .eq(HsayiAuditInfo::getOut_product_id, vo.getOut_product_id())
                .set(HsayiAuditInfo::getAudit_id, audit_info.getAudit_id())
                .set(HsayiAuditInfo::getStatus, spu.getEdit_status())
                .set(HsayiAuditInfo::getAudit_time, audit_info.getAudit_time())
                .set(HsayiAuditInfo::getReject_reason, audit_info.getReject_reason())
                .set(HsayiAuditInfo::getSubmit_time, audit_info.getSubmit_time())
        );
        return spu;
    }

    /**
     * [更新商品]
     *
     * @param param
     * @return com.ads.business.service.data.spu.Spu
     * @author huzhihao
     * @date 2021/12/26 10:40
     */
    public Spu spuUpdate(Spu param) {
        Spu spu = wechatService.spuUpdate(param);
        String out_product_id = spu.getOut_product_id();
        return wechatService.spuGet(out_product_id);
    }

    /**
     * [上架商品]
     *
     * @param outProductId
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2021/12/26 10:40
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean spuListing(String outProductId) {
        return wechatService.spuListing(outProductId);
    }

    /**
     * [下架商品]
     *
     * @param outProductId
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2021/12/26 10:41
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean spuDelisting(String outProductId) {
        return wechatService.spuDelisting(outProductId);
    }

    /**
     * [删除商品]
     *
     * @param outProductId
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2021/12/26 10:41
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean spuDel(String outProductId) {
        return wechatService.spuDel(outProductId);
    }

    /**
     * [提交品牌审核]
     *
     * @param vo
     * @return java.lang.String
     * @author huzhihao
     * @date 2021/12/30 19:51
     */
    @Transactional(rollbackFor = Exception.class)
    public String auditBrand(BrandInfoAddRepVo vo) {
        //获取各个附件
        List<BrandImgDto> imgDtoList = vo.getImgDtoList();
        List<String> orgCodeCertificateList = imgDtoList.stream()
                .filter(t -> t.getType().equals(BrandImgTypeEnum.ORG_CODE_CERTIFICATE.getCode()))
                .map(t -> t.getUrl())
                .collect(Collectors.toList());
        List<String> authorizationList = imgDtoList.stream()
                .filter(t -> t.getType().equals(BrandImgTypeEnum.AUTHORIZATION.getCode()))
                .map(t -> t.getUrl())
                .collect(Collectors.toList());
        List<String> registrationCertificateList = imgDtoList.stream()
                .filter(t -> t.getType().equals(BrandImgTypeEnum.REGISTRATION_CERTIFICATE.getCode()))
                .map(t -> t.getUrl())
                .collect(Collectors.toList());
        List<String> changeCertificateList = imgDtoList.stream()
                .filter(t -> t.getType().equals(BrandImgTypeEnum.CHANGE_CERTIFICATE.getCode()))
                .map(t -> t.getUrl())
                .collect(Collectors.toList());
        List<String> registrationApplicationList = imgDtoList.stream()
                .filter(t -> t.getType().equals(BrandImgTypeEnum.REGISTRATION_APPLICATION.getCode()))
                .map(t -> t.getUrl())
                .collect(Collectors.toList());
        List<String> goodsFormList = imgDtoList.stream()
                .filter(t -> t.getType().equals(BrandImgTypeEnum.GOODS_FORM.getCode()))
                .map(t -> t.getUrl())
                .collect(Collectors.toList());
        //组装实体
        BrandAuditReq brandAuditReq = new BrandAuditReq();
        brandAuditReq.setLicense(orgCodeCertificateList);

        BrandInfo brandInfo = new BrandInfo();
        BeanUtil.copyProperties(vo, brandInfo);
        brandInfo.setTrademark_authorization_period(DateUtil.format(vo.getTrademark_authorization_period(), DatePattern.NORM_DATETIME_FORMAT));
        brandInfo.setTrademark_application_time(DateUtil.format(vo.getTrademark_application_time(), DatePattern.NORM_DATETIME_FORMAT));
        brandInfo.setSale_authorization(authorizationList);
        brandInfo.setTrademark_registration_certificate(registrationCertificateList);
        brandInfo.setTrademark_change_certificate(changeCertificateList);
        brandInfo.setTrademark_registration_application(registrationApplicationList);
        brandInfo.setImported_goods_form(goodsFormList);

        brandAuditReq.setBrand_info(brandInfo);

        BrandAuditPost post = new BrandAuditPost();
        post.setAudit_req(brandAuditReq);

        return wechatService.auditBrand(post);
    }

    /**
     * [提交类目审核]
     *
     * @param vo
     * @return java.lang.String
     * @author huzhihao
     * @date 2021/12/30 19:51
     */
    @Transactional(rollbackFor = Exception.class)
    public String auditCategory(CategoryInfoAddRepVo vo) {

        CatAuditReq catAuditReq = new CatAuditReq();
        catAuditReq.setLicense(Arrays.asList(vo.getLicenseImgs().split(StrUtil.COMMA)));

        CategoryInfo categoryInfo = new CategoryInfo();
        categoryInfo.setLevel1(vo.getLevel1());
        categoryInfo.setLevel2(vo.getLevel2());
        categoryInfo.setLevel3(vo.getLevel3());
        categoryInfo.setCertificate(Arrays.asList(vo.getCertificateImgs().split(StrUtil.COMMA)));

        catAuditReq.setCategory_info(categoryInfo);

        CatAuditPost post = new CatAuditPost();
        post.setAudit_req(catAuditReq);

        return wechatService.auditCategory(post);
    }

    /**
     * [获取订单详情]
     *
     * @param param
     * @return com.ads.business.service.data.order.OrderInfoRes
     * @author huzhihao
     * @date 2021/12/29 14:20
     */
    public OrderInfoRes orderGet(OrderGetReq param) {
        return wechatService.orderGet(param);
    }

    /**
     * [同步订单并发货]
     *
     * @param vo
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2021/12/28 9:12
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean syncOrderAndSend(OrderPayReqVo vo) {

        OrderPayReq orderPayReq = new OrderPayReq();
        BeanUtil.copyProperties(vo, orderPayReq);
        log.info("===>同步订单，参数：{}", orderPayReq);
        Boolean isSuccess = wechatService.orderPay(orderPayReq);
        //支付成功的调用发货
        if (isSuccess && Objects.equals(CodeConstant.INTEGER.INT_1, orderPayReq.getAction_type())) {
            DeliverySendReq deliverySendReq = new DeliverySendReq();
            deliverySendReq.setOut_order_id(orderPayReq.getOut_order_id());
            deliverySendReq.setOpenid(orderPayReq.getOpenid());
            deliverySendReq.setFinish_all_delivery(CodeConstant.INTEGER.INT_1);
            log.info("===>订单发货，参数：{}", deliverySendReq);
            wechatService.deliverySend(deliverySendReq);
        }
        return true;
    }

    /**
     * [验证]
     *
     * @param signature
     * @param timestamp
     * @param nonce
     * @param echostr
     * @return java.lang.String
     * @author huzhihao
     * @date 2021/12/27 14:26
     */
    public String callbackValidate(String signature, String timestamp, String nonce, String echostr) {
        return WeChatHandler.callbackValidate(signature, timestamp, nonce, echostr);
    }

    /**
     * [处理回调]
     *
     * @param map
     * @return void
     * @author huzhihao
     * @date 2021/12/26 10:45
     */
    @Transactional(rollbackFor = Exception.class)
    public void callbackProcess(Map map) {

        String encrypt = (String) map.get(CodeConstant.WECHAT.ENCRYPT);
        String decrypt = null;
        try {
            log.info("===>回调，解码原数据：{}", encrypt);
            decrypt = WeChatHandler.decrypt(encrypt);
            log.info("===>回调，解码数据：{}", decrypt);
        } catch (Exception e) {
        }
        JSONObject jsonObject = JSONUtil.parseObj(decrypt);
        String from_user_name = jsonObject.getStr(CodeConstant.WECHAT.FROM_USER_NAME);
        String create_time = jsonObject.getStr(CodeConstant.WECHAT.CREATE_TIME);
        String event = jsonObject.getStr(CodeConstant.WECHAT.EVENT);
        String eventKey = CodeConstant.REDIS.EVENT +
                StrUtil.UNDERLINE + from_user_name +
                StrUtil.UNDERLINE + create_time;
        boolean hasKey = RedisUtil.hasKey(eventKey);
        if (hasKey) {
            log.info("===>此次回调事件已处理过，不再处理");
            return;
        }
        RedisUtil.set(eventKey, encrypt, 60 * 60 * 24 * 7);
        if (Objects.equals(event, CodeConstant.WECHAT.OPEN_PRODUCT_SPU_AUDIT)) {
            log.info("===>商品审核回调");
            //商品审核回调

            //审核状态,4:已通过,3:拒绝
            String status = jsonObject.getStr(CodeConstant.WECHAT.STATUS);
            String reject_reason = jsonObject.getStr(CodeConstant.WECHAT.REJECT_REASON);
            String out_product_id = jsonObject.getStr(CodeConstant.WECHAT.OUT_PRODUCT_ID);
            // 0初始值 5上架 11自主下架 13违规下架/风控系统下架
            String spu_status = jsonObject.getStr(CodeConstant.WECHAT.SPU_STATUS);
            Spu spu = wechatService.spuGet(out_product_id);
            handleSpu(out_product_id, status, spu_status, spu);

        } else if (Objects.equals(event, CodeConstant.WECHAT.OPEN_PRODUCT_SPU_STATUS_UPDATE)) {
            log.info("商品系统下架回调");
            //商品系统下架回调

            //13，系统下架，5，重新上架
            String status = jsonObject.getStr(CodeConstant.WECHAT.STATUS);
            String reject_reason = jsonObject.getStr(CodeConstant.WECHAT.REJECT_REASON);
            String out_product_id = jsonObject.getStr(CodeConstant.WECHAT.OUT_PRODUCT_ID);
            deListingBySystem(status, reject_reason, out_product_id);

        } else if (Objects.equals(event, CodeConstant.WECHAT.OPEN_PRODUCT_CATEGORY_AUDIT)) {
            log.info("类目审核回调");
            //类目审核回调

            //审核状态,1:已通过,9:拒绝
            String status = jsonObject.getStr(CodeConstant.WECHAT.STATUS);
            String reject_reason = jsonObject.getStr(CodeConstant.WECHAT.REJECT_REASON);
            String audit_id = jsonObject.getStr(CodeConstant.WECHAT.AUDIT_ID);
            //审核类型，本接口固定为2
            jsonObject.getStr(CodeConstant.WECHAT.AUDIT_TYPE);
            //更新状态
            categoryInfoService.update(Wrappers.<HsayiCategoryInfo>lambdaUpdate()
                    .eq(HsayiCategoryInfo::getAudit_id, audit_id)
                    .set(HsayiCategoryInfo::getReject_reason, reject_reason)
                    .set(HsayiCategoryInfo::getStatus, status)
            );

        } else if (Objects.equals(event, CodeConstant.WECHAT.OPEN_PRODUCT_BRAND_AUDIT)) {
            log.info("品牌审核回调");
            //品牌审核回调

            //审核状态,1:已通过,9:拒绝
            String status = jsonObject.getStr(CodeConstant.WECHAT.STATUS);
            String reject_reason = jsonObject.getStr(CodeConstant.WECHAT.REJECT_REASON);
            String audit_id = jsonObject.getStr(CodeConstant.WECHAT.AUDIT_ID);
            String brand_id = jsonObject.getStr(CodeConstant.WECHAT.BRAND_ID);
            //审核类型，本接口固定为1
            jsonObject.getStr(CodeConstant.WECHAT.AUDIT_TYPE);
            //更新状态
            brandInfoService.update(Wrappers.<HsayiBrandInfo>lambdaUpdate()
                    .eq(HsayiBrandInfo::getAudit_id, audit_id)
                    .set(HsayiBrandInfo::getReject_reason, reject_reason)
                    .set(HsayiBrandInfo::getStatus, status)
                    .set(HsayiBrandInfo::getBrand_id, brand_id)
            );
        }

    }

    /**
     * [系统下架]
     *
     * @param status
     * @param reject_reason
     * @param out_product_id
     * @return void
     * @author huzhihao
     * @date 2021/12/28 9:45
     */
    @Transactional(rollbackFor = Exception.class)
    public void deListingBySystem(String status, String reject_reason, String out_product_id) {
        //更新商品数据
        commodityService.update(Wrappers.<HsayiCommodity>lambdaUpdate()
                .eq(HsayiCommodity::getOut_product_id, out_product_id)
                .set(HsayiCommodity::getStatus, status)
        );
        //更新审核数据
        auditInfoService.update(Wrappers.<HsayiAuditInfo>lambdaUpdate()
                .eq(HsayiAuditInfo::getOut_product_id, out_product_id)
                .set(HsayiAuditInfo::getReject_reason, reject_reason)
        );
    }

    /**
     * [处理商品回调]
     *
     * @param out_product_id
     * @param status
     * @param spu_status
     * @param spu
     * @return void
     * @author huzhihao
     * @date 2021/12/27 15:11
     */
    @Transactional(rollbackFor = Exception.class)
    public void handleSpu(String out_product_id, String status, String spu_status, Spu spu) {
        //判断是否通过
        if (SpuEditStatusEnum.SUCCESS.getCode().toString().equals(status)) {
            //通过新增正式表并删除提交表
            //查询提交表数据
            HsayiCommoditySubmit commoditySubmit = commoditySubmitService.getOne(Wrappers.<HsayiCommoditySubmit>lambdaQuery()
                    .eq(HsayiCommoditySubmit::getOut_product_id, out_product_id));
            List<HsayiSkuSubmit> skuSubmitList = skuSubmitService.list(Wrappers.<HsayiSkuSubmit>lambdaQuery()
                    .eq(HsayiSkuSubmit::getOut_product_id, out_product_id));
            List<HsayiSpuImgSubmit> spuImgSubmitList = spuImgSubmitService.list(Wrappers.<HsayiSpuImgSubmit>lambdaQuery()
                    .eq(HsayiSpuImgSubmit::getOut_product_id, out_product_id));
            //装配数据
            HsayiCommodity commodity = new HsayiCommodity();
            BeanUtil.copyProperties(commoditySubmit, commodity);
            commodity.setEdit_status(Integer.valueOf(status));
            commodity.setStatus(Integer.valueOf(spu_status));
            commodity.setCommodity_id(null);
            commodity.setStand_down_time(new Date());
            commodity.setCreate_time(DateUtil.parse(spu.getCreate_time(), DatePattern.NORM_DATETIME_PATTERN));
            commodity.setUpdate_time(DateUtil.parse(spu.getUpdate_time(), DatePattern.NORM_DATETIME_PATTERN));

            List<HsayiSku> skuList = skuSubmitList.stream().map(t -> {
                HsayiSku sku = new HsayiSku();
                BeanUtil.copyProperties(t, sku);
                sku.setSku_id(null);
                return sku;
            }).collect(Collectors.toList());

            List<HsayiSpuImg> spuImgList = spuImgSubmitList.stream().map(t -> {
                HsayiSpuImg spuImg = new HsayiSpuImg();
                BeanUtil.copyProperties(t, spuImg);
                spuImg.setSpu_img_id(null);
                return spuImg;
            }).collect(Collectors.toList());
            //正式表删除数据,除主表外
            //commodityService.remove(Wrappers.<HsayiCommodity>lambdaQuery()
            //        .eq(HsayiCommodity::getOut_product_id, out_product_id));
            skuService.remove(Wrappers.<HsayiSku>lambdaQuery()
                    .eq(HsayiSku::getOut_product_id, out_product_id));
            spuImgService.remove(Wrappers.<HsayiSpuImg>lambdaQuery()
                    .eq(HsayiSpuImg::getOut_product_id, out_product_id));
            //正式表插入数据，主表为更新
            List<HsayiCommodity> commodityList = commodityService.list(Wrappers.<HsayiCommodity>lambdaQuery()
                    .eq(HsayiCommodity::getOut_product_id, out_product_id)
                    .orderByDesc(HsayiCommodity::getCommodity_id));
            if (CollUtil.isNotEmpty(commodityList)) {
                commodity.setCommodity_id(commodityList.get(0).getCommodity_id());
            }
            commodityService.saveOrUpdate(commodity);
            skuService.saveOrUpdateBatch(skuList);
            spuImgService.saveOrUpdateBatch(spuImgList);
            //删除提交表旧数据
            commoditySubmitService.remove(Wrappers.<HsayiCommoditySubmit>lambdaQuery()
                    .eq(HsayiCommoditySubmit::getOut_product_id, out_product_id));
            skuSubmitService.remove(Wrappers.<HsayiSkuSubmit>lambdaQuery()
                    .eq(HsayiSkuSubmit::getOut_product_id, out_product_id));
            spuImgSubmitService.remove(Wrappers.<HsayiSpuImgSubmit>lambdaQuery()
                    .eq(HsayiSpuImgSubmit::getOut_product_id, out_product_id));

        } else if (SpuEditStatusEnum.FAIL.getCode().toString().equals(status)) {
            //不通过仅更新提交表
            commoditySubmitService.update(
                    Wrappers.<HsayiCommoditySubmit>lambdaUpdate()
                            .eq(HsayiCommoditySubmit::getOut_product_id, out_product_id)
                            .set(HsayiCommoditySubmit::getEdit_status, status)
                            .set(HsayiCommoditySubmit::getStatus, spu_status)
            );
        }
        //更新审核数据
        AuditInfo audit_info = spu.getAudit_info();
        auditInfoService.update(Wrappers.<HsayiAuditInfo>lambdaUpdate()
                .eq(HsayiAuditInfo::getOut_product_id, out_product_id)
                .set(HsayiAuditInfo::getAudit_id, audit_info.getAudit_id())
                .set(HsayiAuditInfo::getStatus, spu.getEdit_status())
                .set(HsayiAuditInfo::getAudit_time, audit_info.getAudit_time())
                .set(HsayiAuditInfo::getReject_reason, audit_info.getReject_reason())
                .set(HsayiAuditInfo::getSubmit_time, audit_info.getSubmit_time())
        );
    }

    /**
     * [创建售后]
     *
     * @param vo
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2022/1/7 10:51
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean aftersaleAdd(AftersaleAddReqVo vo) {

        AftersaleAddReq req = new AftersaleAddReq();
        BeanUtil.copyProperties(vo, req);

        List<ProductInfoVo> product_infos = vo.getProduct_infos();
        List<ProductInfo> productInfoList = product_infos.stream().map(t -> {
            ProductInfo productInfo = new ProductInfo();
            BeanUtil.copyProperties(t, productInfo);
            return productInfo;
        }).collect(Collectors.toList());

        req.setProduct_infos(productInfoList);
        log.info("===>创建售后，参数：{}", req);
        return wechatService.aftersaleAdd(req);
    }

    /**
     * [获取商品]
     *
     * @param oid
     * @return java.lang.Object
     * @author huzhihao
     * @date 2022/1/7 10:51
     */
    public Object spuGet(String oid) {
        return wechatService.spuGet(oid);
    }
}