package com.ads.business.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.ads.business.dao.HsayiBrandInfoDao;
import com.ads.business.entity.HsayiBrandImg;
import com.ads.business.entity.HsayiBrandInfo;
import com.ads.business.model.BrandImgDto;
import com.ads.business.model.rep.BrandInfoAddRepVo;
import com.ads.business.model.rep.BrandInfoEditRepVo;
import com.ads.business.model.rep.BrandInfoReqVo;
import com.ads.business.model.resq.BrandInfoDetailRespVo;
import com.ads.business.model.resq.BrandInfoRespVo;
import com.ads.business.service.HsayiBrandImgService;
import com.ads.business.service.HsayiBrandInfoService;
import com.ads.business.service.WechatFacade;
import com.ads.common.data.JsonResult;
import com.ads.common.enums.BrandImgTypeEnum;
import com.ads.common.exception.BusinessException;
import com.ads.common.util.PageUtils;
import com.ads.common.util.UtilMessage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 品牌表 服务实现类
 * </p>
 *
 * @author huzhihao
 * @since 2021-12-24
 */
@Service
public class HsayiBrandInfoServiceImpl extends ServiceImpl<HsayiBrandInfoDao, HsayiBrandInfo> implements HsayiBrandInfoService {


    @Resource
    private HsayiBrandImgService brandImgService;

    @Resource
    private WechatFacade wechatFacade;

    @Override
    public JsonResult findPage(BrandInfoReqVo vo) {
        Long currPage = vo.getPage() == null ? 1 : vo.getPage().longValue();
        Long pageSize = vo.getPageSize() == null ? 10 : vo.getPageSize().longValue();
        Page<BrandInfoRespVo> page = new Page<>(currPage, pageSize);
        Page<BrandInfoRespVo> results = this.baseMapper.findPage(page, vo);
        return JsonResult.success(new PageUtils(results), UtilMessage.GET_MESSAGE_SUCCESS);
    }

    @Override
    public BrandInfoDetailRespVo detail(Long brandInfoId) {
        BrandInfoDetailRespVo detailVo = this.baseMapper.detail(brandInfoId);
        if (ObjectUtil.isEmpty(detailVo)) {
            throw new BusinessException(UtilMessage.BRAND_NOEXIST);
        }
        //查询资质图片
        LambdaQueryWrapper<HsayiBrandImg> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(HsayiBrandImg::getBrand_id, brandInfoId);
        List<HsayiBrandImg> imgs = brandImgService.list(wrapper);
        if (!imgs.isEmpty()) {
            List<BrandImgDto> voList = new ArrayList<>();
            imgs.stream().forEach(t -> {
                BrandImgDto vo = new BrandImgDto();
                vo.setType(t.getType());
                vo.setUrl(t.getUrl());
                voList.add(vo);
            });
            Map<String, List<BrandImgDto>> imgMap = voList.stream().collect(Collectors.groupingBy(BrandImgDto::getType));
            for (BrandImgTypeEnum bit : BrandImgTypeEnum.values()) {
                List<BrandImgDto> list = imgMap.get(bit.getCode());
                if (list == null) {
                    imgMap.put(bit.getCode(), Lists.newArrayList());
                }
            }
            detailVo.setImgMap(imgMap);
        }
        return detailVo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult add(BrandInfoAddRepVo vo) {
        //验证品牌唯一
        LambdaQueryWrapper<HsayiBrandInfo> cwrapper = new LambdaQueryWrapper<>();
        cwrapper.eq(HsayiBrandInfo::getTrademark_registrant_nu, vo.getTrademark_registrant_nu());
        List<HsayiBrandInfo> list = this.list(cwrapper);
        if (!list.isEmpty()) {
            throw new BusinessException(UtilMessage.BRAND_EXIST);
        }
        HsayiBrandInfo newEntity = new HsayiBrandInfo();
        BeanUtil.copyProperties(vo, newEntity);
        String auditId = wechatFacade.auditBrand(vo);
        newEntity.setAudit_id(auditId);
        this.save(newEntity);
        //插入图片
        List<BrandImgDto> imgDtos = vo.getImgDtoList();
        if (!imgDtos.isEmpty()) {
            List<HsayiBrandImg> imgs = Lists.newArrayList();
            imgDtos.stream().forEach(t -> {
                HsayiBrandImg img = new HsayiBrandImg();
                img.setType(t.getType());
                img.setUrl(t.getUrl());
                img.setBrand_id(newEntity.getBrand_info_id());
                imgs.add(img);
            });
            brandImgService.saveBatch(imgs);
        }
        //TODO 推送到微信直播号审核
        return JsonResult.success(UtilMessage.BRAND_SUBMIT_MESSAGE_SUCCESS);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult edit(BrandInfoEditRepVo vo) {
        HsayiBrandInfo oldEntity = this.getById(vo.getBrand_info_id());
        if (ObjectUtil.isEmpty(oldEntity)) {
            throw new BusinessException(UtilMessage.BRAND_NOEXIST);
        }
        LambdaQueryWrapper<HsayiBrandInfo> cwrapper = new LambdaQueryWrapper<>();
        cwrapper.eq(HsayiBrandInfo::getTrademark_registrant_nu, vo.getTrademark_registrant_nu());
        cwrapper.ne(HsayiBrandInfo::getBrand_info_id, vo.getBrand_info_id());
        List<HsayiBrandInfo> list = this.list(cwrapper);
        if (!list.isEmpty()) {
            throw new BusinessException(UtilMessage.BRAND_EXIST);
        }
        HsayiBrandInfo newEntity = new HsayiBrandInfo();
        BeanUtil.copyProperties(vo, newEntity);
        BeanUtil.copyProperties(newEntity, oldEntity);
        String auditId = wechatFacade.auditBrand(vo);
        oldEntity.setAudit_id(auditId);
        oldEntity.setReject_reason("");
        oldEntity.setStatus("0");
        oldEntity.setCreate_time(new Date());
        this.updateById(oldEntity);
        //全量删除
        brandImgService.remove(new LambdaQueryWrapper<HsayiBrandImg>().eq(HsayiBrandImg::getBrand_id, oldEntity.getBrand_info_id()));
        List<BrandImgDto> imgDtos = vo.getImgDtoList();
        if (!imgDtos.isEmpty()) {
            List<HsayiBrandImg> imgs = Lists.newArrayList();
            imgDtos.stream().forEach(t -> {
                HsayiBrandImg img = new HsayiBrandImg();
                img.setType(t.getType());
                img.setUrl(t.getUrl());
                img.setBrand_id(oldEntity.getBrand_info_id());
                imgs.add(img);
            });
            brandImgService.saveBatch(imgs);
        }
        // TODO 推送到微信直播号审核
        return JsonResult.success(UtilMessage.BRAND_EDIT_MESSAGE_SUCCESS);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult deleteByIds(Long[] brandInfoIds) {
        brandImgService.remove(new LambdaQueryWrapper<HsayiBrandImg>().in(HsayiBrandImg::getBrand_id, brandInfoIds));
        this.baseMapper.delete(new LambdaQueryWrapper<HsayiBrandInfo>().in(HsayiBrandInfo::getBrand_info_id, brandInfoIds));
        return JsonResult.success(UtilMessage.DEL_MESSAGE_SUCCESS);
    }
}
