package com.ads.business.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.ads.business.dao.*;
import com.ads.business.entity.*;
import com.ads.business.model.rep.CommodityAddRepVo;
import com.ads.business.model.rep.CommodityEditRepVo;
import com.ads.business.model.rep.CommodityReqVo;
import com.ads.business.model.resq.AuditInfoRespVo;
import com.ads.business.model.resq.CommodityDetailVo;
import com.ads.business.model.resq.CommodityRespVo;
import com.ads.business.model.resq.CommodityTypeCountRespVo;
import com.ads.business.service.HsayiCommodityService;
import com.ads.business.service.HsayiCommoditySubmitService;
import com.ads.business.service.WechatFacade;
import com.ads.common.data.JsonResult;
import com.ads.common.enums.SpuEditStatusEnum;
import com.ads.common.enums.SpuStatusEnum;
import com.ads.common.exception.BusinessException;
import com.ads.common.util.PageUtils;
import com.ads.common.util.UtilConst;
import com.ads.common.util.UtilMessage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 商品表 服务实现类
 * </p>
 *
 * @author huzhihao
 * @since 2021-12-24
 */
@Service
public class HsayiCommodityServiceImpl extends ServiceImpl<HsayiCommodityDao, HsayiCommodity> implements HsayiCommodityService {

    @Resource
    private HsayiAuditInfoDao auditInfoDao;

    @Resource
    private HsayiSkuDao skuDao;

    @Resource
    private HsayiSpuImgDao spuImgDao;

    @Resource
    private HsayiCommoditySubmitDao commoditySubmitDao;

    @Resource
    private HsayiSkuSubmitDao skuSubmitDao;

    @Resource
    private HsayiSpuImgSubmitDao spuImgSubmitDao;

    @Autowired
    private WechatFacade wechatFacade;

    @Autowired
    private HsayiCommoditySubmitService commoditySubmitService;

    @Override
    public JsonResult findPage(CommodityReqVo vo) {
        Long currPage = vo.getPage() == null ? 1 : vo.getPage().longValue();
        Long pageSize = vo.getPageSize() == null ? 10 : vo.getPageSize().longValue();
        if (UtilConst.commodityType.equals(vo.getType())) {
            Page<CommodityRespVo> page = new Page<>(currPage, pageSize);
            Page<CommodityRespVo> results = commoditySubmitDao.findPage(page, vo);
            return JsonResult.success(new PageUtils(results), UtilMessage.GET_MESSAGE_SUCCESS);
        } else {
            Page<CommodityRespVo> page = new Page<>(currPage, pageSize);
            Page<CommodityRespVo> results = this.baseMapper.findPage(page, vo);
            return JsonResult.success(new PageUtils(results), UtilMessage.GET_MESSAGE_SUCCESS);
        }
    }

    @Override
    public CommodityDetailVo detail(String outProductId) {
        //审核通过的商品
        CommodityDetailVo detailVo = this.baseMapper.detail(outProductId);
        if (ObjectUtil.isEmpty(detailVo)) {
            detailVo = new CommodityDetailVo();
        }
        //查询审核数据
        HsayiAuditInfo auditInfo = auditInfoDao.selectOne(new LambdaQueryWrapper<HsayiAuditInfo>().eq(HsayiAuditInfo::getOut_product_id, outProductId));
        AuditInfoRespVo audit = new AuditInfoRespVo();
        if (auditInfo != null) {
            audit.setReject_reason(auditInfo.getReject_reason());
            audit.setOut_product_id(auditInfo.getOut_product_id());
            audit.setStatus(auditInfo.getStatus());
            audit.setStatusName(SpuEditStatusEnum.getByCode(auditInfo.getStatus()));
        }
        detailVo.setAuditInfo(audit);
        //查询资质图片
        LambdaQueryWrapper<HsayiSpuImg> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(HsayiSpuImg::getOut_product_id, outProductId);
        List<HsayiSpuImg> imgs = spuImgDao.selectList(wrapper);
        if (!imgs.isEmpty()) {
            List<String> imgList = imgs.stream().map(HsayiSpuImg::getUrl).collect(Collectors.toList());
            detailVo.setQualificationImgs(imgList);
        }
        return detailVo;
    }

    @Override
    public CommodityDetailVo editDetail(String outProductId) {
        CommodityDetailVo detailVo = this.baseMapper.detail(outProductId);
        if (ObjectUtil.isEmpty(detailVo)) {
            throw new BusinessException(UtilMessage.COMMODITY_NOEXIST);
        }
        //查询资质图片
        LambdaQueryWrapper<HsayiSpuImg> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(HsayiSpuImg::getOut_product_id, outProductId);
        List<HsayiSpuImg> imgs = spuImgDao.selectList(wrapper);
        if (!imgs.isEmpty()) {
            List<String> imgList = imgs.stream().map(HsayiSpuImg::getUrl).collect(Collectors.toList());
            detailVo.setQualificationImgs(imgList);
        }
        return detailVo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult add(CommodityAddRepVo vo) {
        //验证商品id唯一
        LambdaQueryWrapper<HsayiCommodity> cwrapper = new LambdaQueryWrapper<>();
        cwrapper.eq(HsayiCommodity::getOut_product_id, vo.getOut_product_id());
        List<HsayiCommodity> list = this.list(cwrapper);
        if (!list.isEmpty()) {
            throw new BusinessException(UtilMessage.COMMODITY_ID_NOEXIST);
        }
        LambdaQueryWrapper<HsayiCommoditySubmit> cswrapper = new LambdaQueryWrapper<>();
        cswrapper.eq(HsayiCommoditySubmit::getOut_product_id, vo.getOut_product_id());
        List<HsayiCommoditySubmit> lists = commoditySubmitDao.selectList(cswrapper);
        if (!lists.isEmpty()) {
            throw new BusinessException(UtilMessage.COMMODITY_ID_NOEXIST);
        }
        HsayiCommoditySubmit newEntity = new HsayiCommoditySubmit();
        BeanUtil.copyProperties(vo, newEntity);
        commoditySubmitDao.insert(newEntity);
        HsayiSkuSubmit skuEntity = new HsayiSkuSubmit();
        BeanUtil.copyProperties(vo, skuEntity);
        skuEntity.setOut_sku_id(vo.getOut_product_id());
        skuSubmitDao.insert(skuEntity);
        //资质图片
        if (ObjectUtil.isNotEmpty(vo.getQualificationImgs())) {
            String[] arr = vo.getQualificationImgs().split(",");
            for (String str : arr) {
                HsayiSpuImgSubmit spuImg = new HsayiSpuImgSubmit();
                spuImg.setType(UtilConst.qualification);
                spuImg.setOut_product_id(vo.getOut_product_id());
                spuImg.setUrl(str);
                spuImgSubmitDao.insert(spuImg);
            }
        }
        wechatFacade.spuAdd(vo);
        return JsonResult.success(UtilMessage.COMMODITY_SUBMIT_MESSAGE_SUCCESS);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult edit(CommodityEditRepVo vo) {
        //验证商品id唯一
        LambdaQueryWrapper<HsayiCommoditySubmit> cwrapper = new LambdaQueryWrapper<>();
        cwrapper.eq(HsayiCommoditySubmit::getOut_product_id, vo.getOut_product_id());
        cwrapper.ne(HsayiCommoditySubmit::getCommodity_submit_id, vo.getCommodity_id());
        List<HsayiCommoditySubmit> list = commoditySubmitService.list(cwrapper);
        if (!list.isEmpty()) {
            throw new BusinessException(UtilMessage.COMMODITY_ID_NOEXIST);
        }
        LambdaQueryWrapper<HsayiCommoditySubmit> cswrapper = new LambdaQueryWrapper<>();
        cswrapper.eq(HsayiCommoditySubmit::getOut_product_id, vo.getOut_product_id());
        cswrapper.eq(HsayiCommoditySubmit::getEdit_status, SpuEditStatusEnum.FAIL.getCode());
        HsayiCommoditySubmit commoditySubmit = commoditySubmitDao.selectOne(cswrapper);
        if (ObjectUtil.isNotEmpty(commoditySubmit)) {
            HsayiCommodity newEntity = new HsayiCommodity();
            BeanUtil.copyProperties(vo, newEntity);
            BeanUtil.copyProperties(newEntity, commoditySubmit);
            commoditySubmitDao.updateById(commoditySubmit);
            skuSubmitDao.delete(new LambdaQueryWrapper<HsayiSkuSubmit>().eq(HsayiSkuSubmit::getOut_product_id, vo.getOut_product_id()));
            HsayiSkuSubmit skuEntity = new HsayiSkuSubmit();
            BeanUtil.copyProperties(vo, skuEntity);
            skuEntity.setOut_sku_id(vo.getOut_product_id());
            skuSubmitDao.insert(skuEntity);
            spuImgSubmitDao.delete(new LambdaQueryWrapper<HsayiSpuImgSubmit>().eq(HsayiSpuImgSubmit::getOut_product_id, vo.getOut_product_id()));
            if (ObjectUtil.isNotEmpty(vo.getQualificationImgs())) {
                String[] arr = vo.getQualificationImgs().split(",");
                for (String str : arr) {
                    HsayiSpuImgSubmit spuImg = new HsayiSpuImgSubmit();
                    spuImg.setType(UtilConst.qualification);
                    spuImg.setOut_product_id(vo.getOut_product_id());
                    spuImg.setUrl(str);
                    spuImgSubmitDao.insert(spuImg);
                }
            }
        } else {
            HsayiCommoditySubmit newEntity = new HsayiCommoditySubmit();
            BeanUtil.copyProperties(vo, newEntity);
            commoditySubmitDao.insert(newEntity);
            HsayiSkuSubmit skuEntity = new HsayiSkuSubmit();
            BeanUtil.copyProperties(vo, skuEntity);
            skuEntity.setOut_sku_id(vo.getOut_product_id());
            skuSubmitDao.insert(skuEntity);
            //资质图片
            if (ObjectUtil.isNotEmpty(vo.getQualificationImgs())) {
                String[] arr = vo.getQualificationImgs().split(",");
                for (String str : arr) {
                    HsayiSpuImgSubmit spuImg = new HsayiSpuImgSubmit();
                    spuImg.setType(UtilConst.qualification);
                    spuImg.setOut_product_id(vo.getOut_product_id());
                    spuImg.setUrl(str);
                    spuImgSubmitDao.insert(spuImg);
                }
            }
        }
        wechatFacade.spuUpdate(vo);
        return JsonResult.success(UtilMessage.COMMODITY_EDIT_MESSAGE_SUCCESS);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult deleteByIds(String[] outProductIds) {
        skuDao.delete(new LambdaQueryWrapper<HsayiSku>().in(HsayiSku::getOut_product_id, outProductIds));
        spuImgDao.delete(new LambdaQueryWrapper<HsayiSpuImg>().in(HsayiSpuImg::getOut_product_id, outProductIds));
        this.baseMapper.delete(new LambdaQueryWrapper<HsayiCommodity>().in(HsayiCommodity::getOut_product_id, outProductIds));
        for (String outProductId : outProductIds) {
            wechatFacade.spuDel(outProductId);
        }
        return JsonResult.success(UtilMessage.DEL_MESSAGE_SUCCESS);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult changeEditStatus(Long commodityId, Integer editStatus) {
        HsayiCommodity commodity = this.getById(commodityId);
        if (ObjectUtil.isEmpty(commodity)) {
            throw new BusinessException(UtilMessage.COMMODITY_NOEXIST);
        }
        commodity.setStatus(editStatus);
        commodity.setStand_down_time(new Date());
        this.baseMapper.updateById(commodity);
        if (SpuStatusEnum.ON_SHELF.getCode().equals(editStatus)) {//上架
            wechatFacade.spuListing(commodity.getOut_product_id());
            return JsonResult.success(UtilMessage.COMMODITY_UP_MESSAGE_SUCCESS);
        } else {//下架
            wechatFacade.spuDelisting(commodity.getOut_product_id());
            return JsonResult.success(UtilMessage.COMMODITY_DOWN_MESSAGE_SUCCESS);
        }
    }

    @Override
    public List<CommodityTypeCountRespVo> getCountByType() {
        List<CommodityTypeCountRespVo> list = Lists.newArrayList();
        List<CommodityRespVo> results = this.baseMapper.findPage(new CommodityReqVo());
        CommodityReqVo reqVo = new CommodityReqVo();
        reqVo.setType("3");
        List<CommodityRespVo> results2 = commoditySubmitDao.findPage(reqVo);
        Map<Integer, Integer> map = new HashMap<>();
        map.put(1, 0);
        map.put(2, 0);
        map.put(3, 0);
        CommodityTypeCountRespVo respVo1 = new CommodityTypeCountRespVo();
        respVo1.setType(1);
        CommodityTypeCountRespVo respVo2 = new CommodityTypeCountRespVo();
        respVo2.setType(2);
        CommodityTypeCountRespVo respVo3 = new CommodityTypeCountRespVo();
        respVo3.setType(3);
        respVo3.setCount(results2.size());
        if (!results.isEmpty()) {
            Integer count1 = 0;//上架
            Integer count2 = 0;//下架
            for (CommodityRespVo vo : results) {
                if (vo.getStatus() != null) {
                    if (vo.getStatus() == 5) {
                        count1++;
                    } else if (vo.getStatus() == 11 || vo.getStatus() == 13) {
                        count2++;
                    }
                }
            }
            respVo1.setCount(count1);
            respVo2.setCount(count2);
        }
        list.add(respVo1);
        list.add(respVo2);
        list.add(respVo3);
        return list;
    }

    @Override
    public CommodityDetailVo authDetail(String outProductId) {
        CommodityDetailVo detailVo = this.commoditySubmitDao.detail(outProductId);
        if (ObjectUtil.isEmpty(detailVo)) {
            throw new BusinessException(UtilMessage.COMMODITY_NOEXIST);
        }
        //查询资质图片
        LambdaQueryWrapper<HsayiSpuImgSubmit> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(HsayiSpuImgSubmit::getOut_product_id, outProductId);
        List<HsayiSpuImgSubmit> imgs = spuImgSubmitDao.selectList(wrapper);
        if (!imgs.isEmpty()) {
            List<String> imgList = imgs.stream().map(HsayiSpuImgSubmit::getUrl).collect(Collectors.toList());
            detailVo.setQualificationImgs(imgList);
        }
        return detailVo;
    }
}
