package com.ads.business.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.ads.business.dao.HsayiOrderInfoDao;
import com.ads.business.entity.HsayiOrderInfo;
import com.ads.business.model.rep.OrderPayReqVo;
import com.ads.business.service.HsayiOrderInfoService;
import com.ads.business.service.WechatFacade;
import com.ads.business.service.data.order.OrderGetReq;
import com.ads.business.service.data.order.OrderInfoRes;
import com.ads.common.constant.CodeConstant;
import com.ads.common.exception.BusinessException;
import com.ads.common.util.SecurityUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Objects;

import static com.ads.common.util.UtilMessage.*;

/**
 * <p>
 * 订单表 服务实现类
 * </p>
 *
 * @author huzhihao
 * @since 2021-12-24
 */
@Service
public class HsayiOrderInfoServiceImpl extends ServiceImpl<HsayiOrderInfoDao, HsayiOrderInfo> implements HsayiOrderInfoService {

    @Autowired
    private WechatFacade wechatFacade;

    /**
     * [同步订单]
     * 支付订单号，action_type=1且order/add时传的pay_method_type=0时必填
     * 支付完成时间，action_type=1时必填，yyyy-MM-dd HH:mm:ss
     *
     * @param vo
     * @return void
     * @author huzhihao
     * @date 2021/12/28 17:31
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void syncOrder(OrderPayReqVo vo) {
        //获取订单详情
        OrderGetReq orderGetReq = new OrderGetReq();
        orderGetReq.setOut_order_id(vo.getOut_order_id());
        orderGetReq.setOpenid(vo.getOpenid());
        OrderInfoRes orderInfoRes = wechatFacade.orderGet(orderGetReq);
        Integer pay_method_type = orderInfoRes.getOrder().getOrder_detail().getPay_info().getPay_method_type();
        //判断必填项
        if (CodeConstant.INTEGER.INT_1.equals(vo.getAction_type())) {
            if (Objects.equals(pay_method_type, CodeConstant.INTEGER.INT_0)
                    && StrUtil.isBlank(vo.getTransaction_id())) {
                throw new BusinessException(TRANSACTION_ID_EMPTY);
            }
            if (StrUtil.isBlank(vo.getPay_time())) {
                throw new BusinessException(PAY_TIME_EMPTY);
            }
        }
        HsayiOrderInfo orderInfo = new HsayiOrderInfo();
        BeanUtil.copyProperties(vo, orderInfo);
        //向微信同步订单并全部发货
        Boolean flag = wechatFacade.syncOrderAndSend(vo);
        orderInfo.setSync_success(flag);
        this.saveOrUpdate(orderInfo);
    }

    /**
     * [同步订单]
     *
     * @param data
     * @return void
     * @author huzhihao
     * @date 2022/1/10 11:31
     */
    @Override
    public void syncOrder(String data) {
        OrderPayReqVo orderPayReqVo;
        try {
            orderPayReqVo = (OrderPayReqVo) SecurityUtil.decrypt(data, OrderPayReqVo.class);
        } catch (Exception e) {
            throw new BusinessException(VALIDATE_ERROR);
        }
        this.syncOrder(orderPayReqVo);
    }

}
