package com.ads.business.service.impl;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.ads.business.dao.UserDao;
import com.ads.business.entity.User;
import com.ads.business.service.LoginService;
import com.ads.business.service.data.GetLoginInData;
import com.ads.business.service.data.GetLoginOutData;
import com.ads.common.constant.UtilConst;
import com.ads.common.constant.UtilMessage;
import com.ads.common.data.JsonResult;
import com.ads.common.enums.CommonStatusEnum;
import com.ads.common.exception.BusinessException;
import com.ads.common.redis.RedisKeyConstant;
import com.ads.common.redis.RedisManager;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * @Return
 * @Author hou
 * @Date 2021/10/21 16:22
 * @Destription
 */
@Service
@Slf4j
public class LoginServiceImpl implements LoginService {

    @Resource
    private UserDao userDao;

    @Resource
    private RedisManager redisManager;

    @Override
    public JsonResult<GetLoginOutData> login(GetLoginInData inData) {
        log.info("用户登录 data:{}", JSON.toJSONString(inData));
        User user = userDao.selectOne(Wrappers.lambdaQuery(User.class)
                .eq(User::getAccount, inData.getAccount()));
        if (ObjectUtil.isNull(user)) {
            throw new BusinessException(UtilMessage.LOGIN_NO_ACCOUNT);
        }
        if (CommonStatusEnum.NO.getCode().equals(user.getStatus())) {
            throw new BusinessException(UtilMessage.LOGIN_ACCOUNT_INVALID);
        }
        if (!user.getPassword().equals(inData.getPassword())) {
            throw new BusinessException(UtilMessage.LOGIN_PASSWORD_ERROR);
        }
        GetLoginOutData loginOutData = new GetLoginOutData();
        loginOutData.setUserId(user.getUserId());
        loginOutData.setName(user.getName());
        loginOutData.setAccount(user.getAccount());
        String token = IdUtil.simpleUUID();
        loginOutData.setToken(token);
        redisManager.set(RedisKeyConstant.PREFIX + token, JSON.toJSONString(loginOutData), UtilConst.TOKEN_EXPIRE);

        return JsonResult.success(loginOutData, UtilMessage.LOGIN_SUCCESS);
    }

    @Override
    public JsonResult logout(String token) {
        log.info("用户退出登录 token:{}", token);
        redisManager.delete(token);
        return JsonResult.success(null, UtilMessage.LOGIN_OUT_SUCCESS);
    }
}
