package com.ads.common.util;

import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.RSA;
import cn.hutool.json.JSONUtil;
import com.ads.common.exception.BusinessException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import java.lang.reflect.Method;
import java.util.Base64;
import java.util.Objects;

@Slf4j
@Configuration
public class SecurityUtil {

    @Value("${sec.privateKey}")
    String privateKey_;

    @Value("${sec.publicKey}")
    String publicKey_;

    @Value("${sec.sign}")
    String sign_;

    static String privateKey, publicKey, sign;


    /**
     * [初始化]
     *
     * @return void
     * @author huzhihao
     * @date 2022/1/10 10:50
     */
    @PostConstruct
    public void init() {
        privateKey = privateKey_;
        publicKey = publicKey_;
        sign = sign_;
    }

    /**
     * [解码]
     *
     * @param data
     * @param clazz
     * @return java.lang.Object
     * @author huzhihao
     * @date 2022/1/10 10:54
     */
    public static Object decrypt(String data, Class clazz) throws Exception {
        RSA rsa = new RSA(privateKey, null);
        byte[] decrypt = rsa.decrypt(Base64.getDecoder().decode(data), KeyType.PrivateKey);
        Object o = JSONUtil.toBean(StrUtil.str(decrypt, CharsetUtil.CHARSET_UTF_8), clazz);
        Method getSign = o.getClass().getMethod("getSign");
        String signD = (String) getSign.invoke(o);
        if (!Objects.equals(sign, signD)) {
            throw new BusinessException(UtilMessage.VALIDATE_ERROR);
        }
        return o;
    }

    /**
     * [加密]
     *
     * @param data
     * @return java.lang.String
     * @author huzhihao
     * @date 2022/1/10 10:56
     */
    public static String encrypt(Object data) {
        RSA rsa = new RSA(null, publicKey);
        String voStr = JSONUtil.toJsonStr(data);
        byte[] encrypt = rsa.encrypt(StrUtil.bytes(voStr, CharsetUtil.CHARSET_UTF_8), KeyType.PublicKey);
        return Base64.getEncoder().encodeToString(encrypt);
    }

}